package com.joaquim.calvo.serloc.serlocapi.application.service.service;

import com.joaquim.calvo.serloc.serlocapi.application.service.persistance.SerlocServiceRepository;
import com.joaquim.calvo.serloc.serlocapi.domain.service.ServiceType;
import com.joaquim.calvo.serloc.serlocapi.domain.service.request.CreateNewServiceRequest;
import com.joaquim.calvo.serloc.serlocapi.domain.service.service.SerlocService;
import com.joaquim.calvo.serloc.serlocapi.domain.service.Service;
import com.joaquim.calvo.serloc.serlocapi.domain.service.request.GetServicesRequest;
import com.joaquim.calvo.serloc.serlocapi.domain.service.service.ServiceTypeService;
import com.joaquim.calvo.serloc.serlocapi.domain.user.User;
import com.joaquim.calvo.serloc.serlocapi.domain.user.service.UserService;
import org.springframework.transaction.annotation.Transactional;

import javax.inject.Inject;
import javax.inject.Named;
import java.util.List;

@Named
public class SerlocServiceImpl implements SerlocService {

    @Inject
    private SerlocServiceRepository serlocServiceRepository;

    @Inject
    private UserService userService;

    @Inject
    private ServiceTypeService serviceTypeService;

    @Override
    public List<Service> getAvailableServices(GetServicesRequest request) {
        return serlocServiceRepository.getAvailableServices(request);
    }

    @Override
    public List<Service> getServicesForUserByEmail(String userEmail) {
        return serlocServiceRepository.getUserServices(userEmail);
    }

    @Transactional
    @Override
    public Service createNewService(CreateNewServiceRequest request) {

        final User serviceProvider = userService.getUserByEmail(request.getServiceProviderEmail())
                .orElseThrow(RuntimeException::new);

        final ServiceType serviceType = serviceTypeService.getServiceTypeById(request.getServiceTypeId());

        final Integer newServiceId = serlocServiceRepository.getNewId();
        final Service newService = new Service(
                serviceProvider,
                serviceType,
                request.getName(),
                request.getDescription(),
                request.getPrice(),
                request.getLongitude(),
                request.getLatitude()
        );

        newService.setId(newServiceId);

        return serlocServiceRepository.saveNewService(newService);
    }
}
