package com.joaquim.calvo.serloc.serlocapi.application.user.service;

import com.joaquim.calvo.serloc.serlocapi.application.user.persistance.UserRepository;
import com.joaquim.calvo.serloc.serlocapi.domain.user.User;
import com.joaquim.calvo.serloc.serlocapi.domain.user.UserTypes;
import com.joaquim.calvo.serloc.serlocapi.domain.user.request.CreateNewUserRequest;
import com.joaquim.calvo.serloc.serlocapi.domain.user.service.UserService;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.transaction.annotation.Transactional;

import javax.inject.Inject;
import javax.inject.Named;
import java.util.Optional;

@Named
public class UserServiceImpl implements UserService {

    @Inject
    private UserRepository userRepository;

    @Inject
    private PasswordEncoder passwordEncoder;


    @Override
    public User getUserInformation(Integer userId) {
        return userRepository.findUserById(userId);
    }

    @Transactional
    @Override
    public void createNewUser(CreateNewUserRequest request) {

        final String encodedPassword = passwordEncoder.encode(request.getPassword());

        final User newUser = new User(
                request.getEmail(),
                request.getName(),
                encodedPassword,
                UserTypes.NORMAL_USER,
                request.getPhone()
                );

        Optional.ofNullable(request.getDescription())
                .ifPresent(newUser::setDescription);

        userRepository.persistNewUser(newUser);
    }

    @Override
    public Optional<User> getUserByEmail(String email) {
        return userRepository.findUserByEmail(email);
    }
}
