package com.joaquim.calvo.serloc.serlocapi.domain.service;

import com.joaquim.calvo.serloc.serlocapi.domain.user.User;

import javax.persistence.*;
import javax.validation.constraints.NotNull;

@Entity(name = "Service")
@Table(name = "serloc_service")
public class Service {

    @Id
    private Integer id;

    @ManyToOne
    @JoinColumn(name = "service_provider")
    private User serviceProvider;

    @ManyToOne
    @JoinColumn(name = "type_id")
    private ServiceType serviceType;

    @Basic
    private String name;

    @Basic
    private String description;

    @Basic
    private Integer price;

    @Column(name = "location_long")
    private Float longitude;

    @Column(name = "location_alt")
    private Float latitude;

    /**
     * Base service constructor
     * with the base and required fields.
     */
    private Service () {

    }

    /**
     * Complete constructor for a generate
     * a new service definition.
     * @param serviceProvider
     * @param serviceType
     * @param name
     * @param description
     * @param price
     * @param longitude
     * @param latitude
     */
    public Service(
            final User serviceProvider,
            final ServiceType serviceType,
            final String name,
            final String description,
            final Integer price,
            final Float longitude,
            final Float latitude
    ) {

        this.serviceProvider = serviceProvider;
        this.serviceType = serviceType;
        this.name = name;
        this.description = description;
        this.price = price;
        this.longitude = longitude;
        this.latitude = latitude;
    }

    public int getId() {
        return id;
    }

    public User getServiceProvider() {
        return serviceProvider;
    }

    public ServiceType getServiceType() {
        return serviceType;
    }

    public String getName() {
        return name;
    }

    public String getDescription() {
        return description;
    }

    public Integer getPrice() {
        return price;
    }

    public Float getLongitude() {
        return longitude;
    }

    public Float getLatitude() {
        return latitude;
    }

    public void setId(Integer id) {
        this.id = id;
    }
}
