package com.joaquim.calvo.serloc.serlocapi.domain.user;

import com.joaquim.calvo.serloc.serlocapi.domain.service.Service;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import javax.persistence.*;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

@Entity
@Table(name = "user")
public class User implements UserDetails, Serializable {

    @Id
    private int id;

    @Basic
    private String name;

    @Basic
    private String email;

    @Basic
    private String password;

    @Basic
    private String description;

    @Basic
    private String phone;

    @Transient
    private List<Service> wishList;

    /**
     * Default User constructor
     */
    private User() {

    }

    /**
     * Base User constructor with the required fields.
     */
    public User (
            final String email,
            final String name,
            final String password,
            final UserType userType,
            final String phone) {
        this.email = email;
        this.name = name;
        this.password = password;
        this.phone = phone;
        this.description = "";
    }

    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return new ArrayList<>();
    }

    public String getPassword() {
        return password;
    }

    @Override
    public String getUsername() {
        return this.email;
    }

    @Override
    public boolean isAccountNonExpired() {
        return true;
    }

    @Override
    public boolean isAccountNonLocked() {
        return true;
    }

    @Override
    public boolean isCredentialsNonExpired() {
        return true;
    }

    @Override
    public boolean isEnabled() {
        return true;
    }

    public int getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public String getPhone() {
        return phone;
    }

    public void setPhone(String phone) {
        this.phone = phone;
    }


}
