package com.joaquim.calvo.serloc.serlocapi.infraestructure.persistance.configuration;

import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.jdbc.datasource.DriverManagerDataSource;
import org.springframework.orm.jpa.JpaVendorAdapter;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;
import org.springframework.orm.jpa.vendor.HibernateJpaVendorAdapter;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import javax.inject.Inject;
import javax.sql.DataSource;
import java.util.Properties;

@Configuration
@EnableTransactionManagement
public class SerlocJpaConfiguration {

    @Inject
    private Environment environment;

    @Value("${serloc.configuration.database.url}")
    private String dataSourceUrl;

    @Value("${serloc.configuration.database.username}")
    private String dataSourceUsername;

    @Value("${serloc.configuration.database.password}")
    private String dataSourcePassword;


    @Bean
    public DataSource getSerlocDataSource() {
        return new DriverManagerDataSource(
                dataSourceUrl,
                dataSourceUsername,
                dataSourcePassword);
    }

    /**
     * @return jpa EntityManager direct implementation.
     */
    @Bean
    public LocalContainerEntityManagerFactoryBean getJpaEntityManager() {
        final LocalContainerEntityManagerFactoryBean entityManagerFactory =
                new LocalContainerEntityManagerFactoryBean();

        final JpaVendorAdapter vendorAdapter = new HibernateJpaVendorAdapter();

        entityManagerFactory.setDataSource(getSerlocDataSource());
        entityManagerFactory.setPackagesToScan("com.joaquim.calvo.serloc.serlocapi.domain");
        entityManagerFactory.setJpaProperties(getSerlocHibernateProperties());
        entityManagerFactory.setJpaVendorAdapter(vendorAdapter);
        entityManagerFactory.setPersistenceUnitName("serlocPersistenceUnit");
        return entityManagerFactory;
    }

    /**
     * Hibernate properties configuration.
     * @return hibernate properties configuration.
     */
    public Properties getSerlocHibernateProperties() {
        Properties hibernateProperties = new Properties();

        hibernateProperties.setProperty(
                "hibernate.dialect",
                "org.hibernate.dialect.MySQL5Dialect"
        );

        hibernateProperties.setProperty(
                "hibernate.globally_quoted_identifiers",
                "true"
        );

        return hibernateProperties;
    }
}
