package com.joaquim.calvo.serloc.serlocapi.infraestructure.persistance.repository.service;

import com.joaquim.calvo.serloc.serlocapi.domain.service.Service;
import com.joaquim.calvo.serloc.serlocapi.domain.service.request.GetServicesRequest;
import com.joaquim.calvo.serloc.serlocapi.infraestructure.persistance.repository.SerlocServiceRepository;

import javax.inject.Named;
import javax.persistence.Query;
import javax.persistence.TypedQuery;
import java.util.List;

@Named
public class HibernateSerlocServiceServiceRepository
        extends SerlocServiceRepository
        implements com.joaquim.calvo.serloc.serlocapi.application.service.persistance.SerlocServiceRepository {

    @Override
    public List<Service> getAvailableServices(GetServicesRequest request) {
        final TypedQuery<Service> query = getSerlocPersistence().createQuery("FROM Service service" +
                " WHERE " +
                    "((service.longitude > :minLongitude " +
                    " AND service.longitude < :maxLongitude " +
                    " AND service.latitude > :minLatitude " +
                    " AND service.latitude < :maxLatitude)" +
                    " OR " +
                    " (service.longitude = :minLongitude " +
                    " OR service.longitude = :maxLongitude " +
                    " OR service.latitude = :minLatitude " +
                    " OR service.latitude = :maxLatitude))" +
                " AND " +
                " service.serviceType.id = :serviceTypeId ", Service.class)
                .setParameter("minLongitude", request.getMinLongitude())
                .setParameter("maxLongitude", request.getMaxLongitude())
                .setParameter("minLatitude", request.getMinLatitude())
                .setParameter("maxLatitude", request.getMaxLatitude())
                .setParameter( "serviceTypeId", request.getServiceType());

        return query.getResultList();
    }

    @Override
    public List<Service> getUserServices(String email) {
        final TypedQuery<Service> query = getSerlocPersistence().createQuery(
                " SELECT service " +
                        " FROM Service service" +
                        " INNER JOIN service.serviceProvider as provider" +
                        " WHERE provider.email = :serviceProviderEmail ", Service.class)
                .setParameter("serviceProviderEmail", email);

        return query.getResultList();

    }

    @Override
    public Service saveNewService(Service newService) {
        getSerlocPersistence().persist(newService);
        return newService;
    }

    @Override
    public Integer getNewId() {
        final Query getLastIdQuery = getSerlocPersistence().createNativeQuery("SELECT MAX(id) FROM serloc_service;");
        return ((Integer)getLastIdQuery.getResultList().get(0)) + 1;
    }
}
