package com.joaquim.calvo.serloc.serlocapi.application.user;

import com.joaquim.calvo.serloc.serlocapi.application.user.persistance.UserRepository;
import com.joaquim.calvo.serloc.serlocapi.application.user.service.UserServiceImpl;
import com.joaquim.calvo.serloc.serlocapi.domain.user.request.CreateNewUserRequest;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.springframework.security.crypto.password.PasswordEncoder;

public class UserServiceImplTest {

    @InjectMocks
    private UserServiceImpl userService;

    @Mock
    private UserRepository userRepository;

    @Mock
    private PasswordEncoder passwordEncoder;

    @Before
    public void init() {
        MockitoAnnotations.initMocks(this);
    }

    @Test
    public void createNewUserMustEncodeTheReceivedPassword() {
        mockAllDependenciesForCreateNewuser();
        final CreateNewUserRequest basicRequest = getBaseCreateRequest();

        userService.createNewUser(basicRequest);

        Mockito.verify(passwordEncoder, Mockito.times(1)).
                encode(basicRequest.getPassword());
    }

    @Test
    public void createNewUserMustCallRepositoryToPersistNewUserDefinition() {
        mockAllDependenciesForCreateNewuser();
        final CreateNewUserRequest request = getBaseCreateRequest();

        userService.createNewUser(request);

        Mockito.verify(userRepository, Mockito.times(1))
                .persistNewUser(Mockito.any());
    }

    @Test
    public void getUserInformationMustCallRepositoryLayer() {

        userService.getUserInformation(1);

        Mockito.verify(userRepository, Mockito.times(1))
                .findUserById(1);

    }

    @Test
    public void getUserByEmailMustCallRepositoryLayer() {

        userService.getUserByEmail("test1234@gmail.com");

        Mockito.verify(userRepository, Mockito.times(1))
                .findUserByEmail("test1234@gmail.com");
    }

    /**
     * Function which mock all the dependencies im
     */
    private void mockAllDependenciesForCreateNewuser() {
        Mockito.when(passwordEncoder.encode(Mockito.any()))
                .thenReturn("anyEncoderPassword");
    }

    /**
     * Obtain a default {@link CreateNewUserRequest} instance for test purpouse.
     * @return the default request.
     */
    private CreateNewUserRequest getBaseCreateRequest() {
        final CreateNewUserRequest request = new CreateNewUserRequest();

        request.setDescription("New user description");
        request.setEmail("newUserEmail@gmail.com");
        request.setName("Username usersurname");
        request.setPassword("test1234");
        request.setPhone("633651614");

        return request;
    }
}
